//(c)2010 by Texas Instruments Incorporated, All Rights Reserved.
/*  
 * ======== main.c ========
 * OLIMEXINO-5510 ADC Example

 * Designed by: OLIMEX Team
 *
 * ----------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 * ---------------------------------------------------------------------------*/
#include <intrinsics.h>
#include <string.h>
#include <stdio.h>

#include "adc.h"

#include "USB_config/descriptors.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB-specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"

#include "USB_API/USB_CDC_API/UsbCdc.h"
#include "usbConstructs.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);
volatile BYTE bCDCDataReceived_event = FALSE;   //Flag set by event handler to indicate data has been received into USB buffer
void Init_Timer1 (void);
void Low_Power_Entering(void);

//=====================Definitions=========================
#define     BUFFER_SIZE     130
#define     STR_SIZE        100

#define     TIMER1_PERIOD   4000              //timer1 period in 1/32768 Hz
#define     LED1_ON          PJOUT |= BIT3
#define     LED1_OFF         PJOUT &= ~BIT3

#define     LED1_TOGGLE      PJOUT ^= BIT3

#define     BUT             (P2IN & BIT0)

#define     PRESSED         0
#define     RELEASED        1

#define     BASE_MEASURE    937
#define     BASE_OFFSET     30



char dataBuffer[BUFFER_SIZE];
char nl[2] = "\n";
char Test_result_counter = 0;
char str[STR_SIZE];
int temp = 0;
char str_temp[8];
unsigned char k = 0;
extern volatile unsigned int results[Num_of_Results];

/*----------------------------------------------------------------------------+
 | Main Routine                                                                |
 +----------------------------------------------------------------------------*/
 VOID main (VOID)
{

    WDTCTL = WDTPW + WDTHOLD;     //Stop watchdog timer
    
// Init_Timer1();                         

    Init_Ports();                                //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                                               //Init clocks
     
//  ADC testing    

   ADC_Init();
   __enable_interrupt();                           //Enable interrupts globally
//  while(1);
   
            	
    USB_init();                 //Init USB

 //Enable various USB event handling routines

    USB_setEnabledEvents(
        kUSB_VbusOnEvent + kUSB_VbusOffEvent + kUSB_receiveCompletedEvent
        + kUSB_dataReceivedEvent + kUSB_UsbSuspendEvent + kUSB_UsbResumeEvent +
        kUSB_UsbResetEvent);

 //See if we're already attached physically to USB, and if so, connect to it
 //Normally applications don't invoke the event handlers, but this is an exception.
    if (USB_connectionInfo() & kUSB_vbusPresent){
        if (USB_enable() == kUSB_succeed){
            USB_reset();
            USB_connect();
        }
    }

    __enable_interrupt();                           //Enable interrupts globally
    while (1)
    {
        BYTE ReceiveError = 0, SendError = 0;
        WORD count;
 //Check the USB state and loop accordingly
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:

                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:

               
            case 0:
                  {                                                                                
                    if (bCDCDataReceived_event)
                    {   
                      count = cdcReceiveDataInBuffer((BYTE*)dataBuffer,BUFFER_SIZE,CDC0_INTFNUM);
                      bCDCDataReceived_event = FALSE;             
                      temp = (unsigned long)results[0];
                      temp = temp*4.58;                          //approximately 4.58 = (1500/1024)*(1470/470) - Battery voltage      
                      sprintf(str_temp,"%d mV\n\r",temp);            //prepare string                                                                 
                      if (cdcSendDataInBackground(str_temp,strlen(str_temp),CDC0_INTFNUM,1))
                      {   	
                       SendError = 0x01;                             //Something went wrong -- exit
                       break;
                      }                                                                                                               
                    }                                                                                                                                            
                    break;
                  }                                                                                                                                                                 
//--------------------------end ADC test -------------------------------------------------------                               
                
                break;                                        //break of USB case  
            default:;
        }
    }  
}                              
/*  
 * ======== Init_Clock ========
 */
VOID Init_Clock (VOID)
{
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x )
			P5SEL |= 0x0C;                                      //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif
//--------------------------------------------------------------                        
      int timeout = 5000;

      P5DIR &= ~(BIT5 + BIT4);
      P5SEL |= BIT5 + BIT4;           //enable XT1 pins
      
      UCSCTL6 |= XCAP0 + XCAP1;
      UCSCTL6 = UCSCTL6 & (~XT1OFF);            //set XT1OFF=0 (bit0 = 0) --> enable XT1
                        
                        
      //use REFO for FLL 
      UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
      //use XT1CLK for ACLK
      UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__XT1CLK);      
      
      do
      {
        if (!(UCSCTL7 & XT1LFOFFG)) break;
        UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
        SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
        timeout--;
      }
      while ((SFRIFG1 & OFIFG) && timeout);      // Test oscillator fault flag

      UCSCTL6 = (UCSCTL6 & ~(1<<6)) & ~(1<<7);  //XT1Drive = 00; --> oscilator range 4-8 MHz
 //--------------------------------------------------------------     
      
      
      
      //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
      Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
     
      XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
      
      
    } 
	else {
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x10;                                      //enable XT1 pins
		#endif
        //Use the REFO oscillator to source the FLL and ACLK
        UCSCTL3 = SELREF__REFOCLK;
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

        XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
        
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0x00;
    P2REN |= BIT0;                //Enable BUT2 pullup
    P2OUT = 0x01;
    P2DIR = 0x00;      
    P4OUT = 0x00;
    P4DIR = 0x00;
    P5OUT = 0x00;
    P5DIR = 0x00;
    P6OUT = 0x00;
    P6DIR = 0x00;
    PJDIR = BIT2+BIT3+BIT1;
    PJOUT |= BIT1;
    PJOUT &= ~(BIT2+BIT3);
}

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
            SYSBERRIV = 0;                                      //clear bus error flag
            USB_disable();                                      //Disable
    }
}

/*  
 * ======== Init_Timer1 ========
 */
void Init_Timer1 (void)
{  
   TA1CTL |= TASSEL0 + MC0 + TACLR;              //ACLK for T1 source
                                                 //Up mode: Timer counts up to TAxCCR0
                                                 //reset timer A1                                         
   TA1CCTL0 |= CCIE;                             //Interrupt enabled
   TA1CCR0 = TIMER1_PERIOD;   
}

